@@ -1,26 +1,28 @@
-#![allow(unsafe_code)]
-#![warn(missing_docs)]
-#![warn(clippy::pedantic)]
-
-#![no_std]
-
-//! C FFI opaque pointers.
+//! # FFI opaque pointers.
 //! 
 //! FFI to use Rust objects from C as opaque pointer.
 
-#[cfg(not(feature = "std"))]
+#![allow(unsafe_code)]
+#![warn(missing_docs)]
+#![warn(clippy::pedantic)]
+#![deny(clippy::complexity)]
+#![deny(clippy::cognitive_complexity)]
+#![allow(clippy::needless_return)]
+
+#![no_std]
+
+#[cfg(all(feature = "alloc", not(feature = "std")))]
 extern crate alloc;
-#[cfg(not(feature = "std"))]
+#[cfg(all(feature = "alloc", not(feature = "std")))]
 use alloc::boxed::Box;
+
 #[cfg(feature = "std")]
 extern crate std;
 #[cfg(feature = "std")]
 use std::boxed::Box;
 
-#[cfg(feature = "std")]
-use std::os::raw::c_char;
-#[cfg(feature = "std")]
-use std::ffi::CStr;
+#[cfg(all(feature = "std", feature = "c-types"))]
+pub mod c;
 
 /// Panic if a pointer is null.
 #[inline]
@@ -31,9 +33,10 @@ fn panic_if_null<T>(pointer: *const T) {
 }
 
 /// Convert type to raw pointer.
+#[cfg(any(feature = "alloc", feature = "std"))]
 #[inline]
 pub fn raw<T>(data: T) -> *mut T {
-    Box::into_raw(Box::new(data))
+    return Box::into_raw(Box::new(data));
 }
 
 /// Free pointer to type.
@@ -41,6 +44,7 @@ pub fn raw<T>(data: T) -> *mut T {
 /// # Safety
 /// 
 /// Never call it twice. That could produce a HEAP error that produce a crash.
+#[cfg(any(feature = "alloc", feature = "std"))]
 #[inline]
 pub unsafe fn free<T>(pointer: *mut T) {
     if pointer.is_null() {
@@ -59,12 +63,13 @@ pub unsafe fn free<T>(pointer: *mut T) {
 /// # Safety
 /// 
 /// Never call it twice. That could produce a HEAP error that produce a crash.
+#[cfg(any(feature = "alloc", feature = "std"))]
 #[inline]
 pub unsafe fn own_back<T>(pointer: *mut T) -> T {
     panic_if_null(pointer);
     // CAUTION: this is unsafe
     let boxed = Box::from_raw(pointer);
-    *boxed
+    return *boxed;
 }
 
 /// Convert raw pointer to type to type reference.
@@ -76,7 +81,7 @@ pub unsafe fn own_back<T>(pointer: *mut T) -> T {
 pub unsafe fn object<'a, T>(pointer: *const T) -> &'a T {
     panic_if_null(pointer);
     // CAUTION: this is unsafe
-    &*pointer
+    return &*pointer;
 }
 
 /// Convert raw pointer to type into type mutable reference.
@@ -88,24 +93,5 @@ pub unsafe fn object<'a, T>(pointer: *const T) -> &'a T {
 pub unsafe fn mut_object<'a, T>(pointer: *mut T) -> &'a mut T {
     panic_if_null(pointer);
     // CAUTION: this is unsafe
-    &mut *pointer
-}
-
-/// Reference to a C string.
-/// 
-/// # Safety
-/// 
-/// The pointer must be a valid reference to that value with that type.
-/// 
-/// # Panics
-/// 
-/// This could panic if the C string is not a valid UTF-8 string.
-#[cfg(feature = "std")]
-#[must_use]
-#[inline]
-pub unsafe fn ref_str<'a>(string: *const c_char) -> &'a str {
-    panic_if_null(string);
-    // CAUTION: this is unsafe
-    let string = CStr::from_ptr(string);
-    string.to_str().expect("Invalid UTF-8 string from C or C++ code")
+    return &mut *pointer;
 }
